/*
** glbucket.c
**
** Written by Ian Buck
** 6/15/2000
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#include <math.h>
#include <float.h>
#include "glcontext.h"
#include "glerror.h"
#include "glhw.h"

#ifdef WINDOWS
#pragma warning( disable : 4514 )  
#endif 

int numprojectors;
GLrectf p[32];
int valid[32];
GLrecti vbounds[32];
GLbitvalue pbit[32];
GLint __bktactive = 0;
GLint __bktalwaysbroadcast = 0;
GLint __bktbroadcast = 0;
GLint __bktnobuffer = 0;
GLfloat __bktobjectbox[6];
GLfloat __bktscreenbox[6];
GLbitvalue __retval = 0;

GLint __tmp_max;
GLint __tmp_count;
GLvectorf __tmp_bmin;
GLvectorf __tmp_bmax;

GLvectorf maxvector;
GLvectorf minvector;

void
__glstate_SetBroadcast( GLint state )
{
	__bktbroadcast = state;
}

void
__glbucket_init(GLbucketstate *b, GLconfig *c) {
	int i;

	maxvector.x = FLT_MAX;
	maxvector.y = FLT_MAX;
	maxvector.z = FLT_MAX;
	maxvector.w = FLT_MAX;

	minvector.x = -FLT_MAX;
	minvector.y = -FLT_MAX;
	minvector.z = -FLT_MAX;
	minvector.w = -FLT_MAX;

	b->hint = 0;
	b->active = 0;
	__bktactive = 0;
		
	b->count = c->vertex_count;
	b->max = c->vertex_max;
	b->bmin = c->bounds_min;
	b->bmax = c->bounds_max;

	if (!b->count)
		b->count = &__tmp_count;
	if (!b->max)
		b->max = &__tmp_max;
	if (!b->bmin)
		b->bmin = &__tmp_bmin;
	if (!b->bmax)
		b->bmax = &__tmp_bmax;

	*b->bmin = maxvector;
	*b->bmax = minvector;
	*b->count = 0;
	*b->max = c->bucket_size;

	if (c->numprojectors == 0 || c->broadcast || c->pack_only )
		__bktalwaysbroadcast = 1;

	if (c->numprojectors == 0)
		__bktnobuffer = 1;

	if (c->numprojectors > 32)
		__glerror(__LINE__, __FILE__, GL_NO_ERROR, 
		"Cannot support more than 32 outputs");
	
	numprojectors = c->numprojectors;
	for (i=0; i<c->numprojectors; i++) {
		vbounds[i] = c->bounds[i];
		p[i].x1 = (GLfloat) c->bounds[i].x1;
		p[i].x2 = (GLfloat) c->bounds[i].x2;
		p[i].y1 = (GLfloat) c->bounds[i].y1;
		p[i].y2 = (GLfloat) c->bounds[i].y2;
	
		p[i].x1 /= c->viewport.x2 / 2.0f;
		p[i].y1 /= c->viewport.y2 / 2.0f;
		p[i].x1 -= 1.0f;
		p[i].y1 -= 1.0f;
		p[i].x2 /= c->viewport.x2 / 2.0f;
		p[i].y2 /= c->viewport.y2 / 2.0f;
		p[i].x2 -= 1.0f;
		p[i].y2 -= 1.0f;
		pbit[i] = 1 << i;
		valid[i] = 1;
	}

	b->flush = c->flush;
}

void
__glbucket_destroy(GLbucketstate *) {
}

void __glbucket_bounds(GLint x, GLint y, GLint w, GLint h) {
	GLint i;
	GLrecti v;
	v.x1 = x;
	v.x2 = x+w;
	v.y1 = y;
	v.y2 = y+h;

	for (i=0; i<numprojectors; i++) {
		GLrecti b = v;
		GLrecti s = vbounds[i];

		/* Clamp to projector bounds */
		if (b.x1 < s.x1) b.x1 = s.x1;
		if (b.x1 > s.x2) b.x1 = s.x2;
		if (b.x2 < s.x1) b.x2 = s.x1;
		if (b.x2 > s.x2) b.x2 = s.x2;
		if (b.y1 < s.y1) b.y1 = s.y1;
		if (b.y1 > s.y2) b.y1 = s.y2;
		if (b.y2 < s.y1) b.y2 = s.y1;
		if (b.y2 > s.y2) b.y2 = s.y2;

		if (b.x1 == b.x2 ||
			b.y1 == b.y2) {
			valid[i] = 0;
			continue;
		}
		valid[i] = 1;

		/* Compute normalized viewports */
		p[i].x1 = ((GLfloat) (b.x1 - v.x1)) / ((GLfloat) (v.x2 - v.x1));
		p[i].x2 = ((GLfloat) (b.x2 - v.x1)) / ((GLfloat) (v.x2 - v.x1));
		p[i].y1 = ((GLfloat) (b.y1 - v.y1)) / ((GLfloat) (v.y2 - v.y1));
		p[i].y2 = ((GLfloat) (b.y2 - v.y1)) / ((GLfloat) (v.y2 - v.y1));
		
		/* Convert to clip coordinates */
		p[i].x1 = p[i].x1*2.0f - 1.0f;	
		p[i].x2 = p[i].x2*2.0f - 1.0f;	
		p[i].y1 = p[i].y1*2.0f - 1.0f;	
		p[i].y2 = p[i].y2*2.0f - 1.0f;	
	}	
}

GLrectf * __glbucket_getclip(GLbitvalue bitID) {
	GLbitvalue a;
	static GLrectf fullbounds =  { -1.0f, 1.0f, -1.0f, 1.0f };
	int i;
	if (!bitID)
		UNIMPLEMENTED();

	if (!numprojectors)
		return &fullbounds;

	for (a=1, i=0; !(bitID&a); i++, a=1<<i);
	return p+i;
}

void
__glbucket_begin() {
	GLcontext *g = GetCurrentContext();
	GLtransstate *t = &(g->trans);
	GLbucketstate *bkt = &(g->bucket);

	if (!t->transformvalid)
		__gltrans_UpdateTransform(t);
	
	if (__bktnobuffer)
		bkt->flush(g);

	bkt->active=1;
	__bktactive=1;
}

static 
float _vmult(float *m, float x, float y, float z) {
	float a;
	a =  *m*x;	m += 4;
	a += *m*y;	m += 4;
	a += *m*z;	m += 4;
	a += *m;
	return a;
}

void
__glbucket_drawbbox(void) {
	
	GLfloat xmin = __bktobjectbox[0];
	GLfloat xmax = __bktobjectbox[1];
	GLfloat ymin = __bktobjectbox[2];
	GLfloat ymax = __bktobjectbox[3];
	GLfloat zmin = __bktobjectbox[4];
	GLfloat zmax = __bktobjectbox[5];

	static int init=0;
	static GLfloat c[32][3];
	unsigned int i;
	GLbitvalue a;
	GLfloat outcolor[3] = {0.0f, 0.0f, 0.0f};
	GLfloat tot;

	if (!init) {
		for (i=0; i<32; i++) {
			c[i][0] = (GLfloat) rand();
			c[i][1] = (GLfloat) rand();
			c[i][2] = (GLfloat) rand();
			tot = (GLfloat) sqrt (c[i][0]*c[i][0] + c[i][1]*c[i][1] + c[i][2]*c[i][2]);
			c[i][0] /= tot;
			c[i][1] /= tot;
			c[i][2] /= tot;
		}
		init = 1;
	}		

	tot = 0.0f;
	for (i=0, a=1; i<32; i++, a*=2)
		if (__retval&a) {
			outcolor[0] += c[i][0];
			outcolor[1] += c[i][1];
			outcolor[2] += c[i][2];
			tot+=1.0f;
		}
	outcolor[0] /= tot;
	outcolor[1] /= tot;
	outcolor[2] /= tot;

	__glhw_PushAttrib(GL_CURRENT_BIT | GL_ENABLE_BIT);
	__glhw_Disable(GL_TEXTURE_2D);
	__glhw_Disable(GL_TEXTURE_1D);
	__glhw_Disable(GL_LIGHTING);
	__glhw_Disable(GL_BLEND);
	__glhw_Disable(GL_ALPHA_TEST);
	__glhw_Disable(GL_DEPTH_TEST);
	__glhw_Disable(GL_FOG);
	__glhw_Disable(GL_STENCIL_TEST);
	__glhw_Disable(GL_SCISSOR_TEST);
	__glhw_Disable(GL_LOGIC_OP);

	__glhw_Color3fv(outcolor);
	__glhw_Begin(GL_LINE_LOOP);
	__glhw_Vertex3f(xmin, ymin, zmin);
	__glhw_Vertex3f(xmin, ymin, zmax);
	__glhw_Vertex3f(xmin, ymax, zmax);
	__glhw_Vertex3f(xmin, ymax, zmin);
	__glhw_End();
	__glhw_Begin(GL_LINE_LOOP);
	__glhw_Vertex3f(xmax, ymin, zmin);
	__glhw_Vertex3f(xmax, ymin, zmax);
	__glhw_Vertex3f(xmax, ymax, zmax);
	__glhw_Vertex3f(xmax, ymax, zmin);
	__glhw_End();
	__glhw_Begin(GL_LINE_LOOP);
	__glhw_Vertex3f(xmin, ymin, zmin);
	__glhw_Vertex3f(xmax, ymin, zmin);
	__glhw_Vertex3f(xmax, ymax, zmin);
	__glhw_Vertex3f(xmin, ymax, zmin);
	__glhw_End();
	__glhw_Begin(GL_LINE_LOOP);
	__glhw_Vertex3f(xmin, ymin, zmax);
	__glhw_Vertex3f(xmax, ymin, zmax);
	__glhw_Vertex3f(xmax, ymax, zmax);
	__glhw_Vertex3f(xmin, ymax, zmax);
	__glhw_End();

	__glhw_PopAttrib();
}

GLbitvalue
__glbucket_flush () {
	GLcontext *g = GetCurrentContext();
	GLbucketstate *bkt = &(g->bucket);
	GLtransstate *t = &(g->trans);
    GLmatrix *m = &(t->transform);
	float x[8], y[8], z[8], w[8];
	float xmin, ymin, xmax, ymax, zmin, zmax;
	int i,j;

	GLbitvalue retval;
/*  Here is the arragement of the bounding box
**  0 --- 1
**  |\    .\
**  | 2 --- 3 
**  | |   . |
**  | |   . |
**  4.|...5 |
**   \|    .|
**    6 --- 7
**  c array contains the edge connectivitiy list
*/
	static const int c[8][3] = {	{1, 2, 4}, {0, 3, 5}, {0, 3, 6}, {1, 2, 7},
									{0, 5, 6}, {1, 4, 7}, {2, 4, 7}, {3, 5, 6} };

	if (__bktalwaysbroadcast) {
		if (!g->current.beginend) {
			bkt->active = 0;
			__bktactive = 0;
		}
		__retval = GLBITS_ONES;
		return GLBITS_ONES;
	}


	/* If the screen space hint is turned on skip the transform */
	if (bkt->hint == WIREGL_SCREEN_BBOX_HINT) {
		xmin = bkt->bmin->x;
		ymin = bkt->bmin->y;
		xmax = bkt->bmax->x;
		ymax = bkt->bmax->y;
		retval = 0;
		for (i=0; i<numprojectors; i++) {
			if (xmin < p[i].x2 && xmax > p[i].x1 &&
				ymin < p[i].y2 && ymax > p[i].y1) {
				retval |= 1 << i;
			}
		}
		*bkt->count = 0;
		if (!g->current.beginend) {
			bkt->active = 0;
			__bktactive = 0;
		}
		__retval = retval;
		return retval;
	}

	// if were in broadcast mode, return all ones;
	// If we're defining a display list, it also goes to everybody.
	if (bkt->hint != WIREGL_OBJECT_BBOX_HINT &&
		(__bktbroadcast ||
		__bktalwaysbroadcast ||
		g->lists.newend ||
		bkt->hint == WIREGL_BROADCAST_HINT)) {
		if (!g->current.beginend) {
			bkt->active = 0;
			__bktactive = 0;
		}
		*bkt->count = 0;
		if (bkt->hint != WIREGL_OBJECT_BBOX_HINT) {
			*bkt->bmin = maxvector;
			*bkt->bmax = minvector;
		}
		__retval = GLBITS_ONES;
		return GLBITS_ONES;
	}

	// If there are no vertices, return.
	if (!*(bkt->count)) {
		if (!g->current.beginend) {
			bkt->active = 0;
			__bktactive = 0;
		}
		__retval = 0;
		return 0;
	}

	/* Load the bounding box */
	xmin = bkt->bmin->x;
	ymin = bkt->bmin->y;
	zmin = bkt->bmin->z;
	xmax = bkt->bmax->x;
	ymax = bkt->bmax->y;
	zmax = bkt->bmax->z;

	/* Copy for export */
	__bktobjectbox[0]= xmin;
	__bktobjectbox[1]= xmax;
	__bktobjectbox[2]= ymin;
	__bktobjectbox[3]= ymax;
	__bktobjectbox[4]= zmin;
	__bktobjectbox[5]= zmax;

	//Now transform the bounding box points
	x[0] = _vmult(&(m->m00), xmin, ymin, zmin);
	x[1] = _vmult(&(m->m00), xmax, ymin, zmin);
	x[2] = _vmult(&(m->m00), xmin, ymax, zmin);
	x[3] = _vmult(&(m->m00), xmax, ymax, zmin);
	x[4] = _vmult(&(m->m00), xmin, ymin, zmax);
	x[5] = _vmult(&(m->m00), xmax, ymin, zmax);
	x[6] = _vmult(&(m->m00), xmin, ymax, zmax);
	x[7] = _vmult(&(m->m00), xmax, ymax, zmax);

	y[0] = _vmult(&(m->m01), xmin, ymin, zmin);
	y[1] = _vmult(&(m->m01), xmax, ymin, zmin);
	y[2] = _vmult(&(m->m01), xmin, ymax, zmin);
	y[3] = _vmult(&(m->m01), xmax, ymax, zmin);
	y[4] = _vmult(&(m->m01), xmin, ymin, zmax);
	y[5] = _vmult(&(m->m01), xmax, ymin, zmax);
	y[6] = _vmult(&(m->m01), xmin, ymax, zmax);
	y[7] = _vmult(&(m->m01), xmax, ymax, zmax);

	z[0] = _vmult(&(m->m02), xmin, ymin, zmin);
	z[1] = _vmult(&(m->m02), xmax, ymin, zmin);
	z[2] = _vmult(&(m->m02), xmin, ymax, zmin);
	z[3] = _vmult(&(m->m02), xmax, ymax, zmin);
	z[4] = _vmult(&(m->m02), xmin, ymin, zmax);
	z[5] = _vmult(&(m->m02), xmax, ymin, zmax);
	z[6] = _vmult(&(m->m02), xmin, ymax, zmax);
	z[7] = _vmult(&(m->m02), xmax, ymax, zmax);

	w[0] = _vmult(&(m->m03), xmin, ymin, zmin);
	w[1] = _vmult(&(m->m03), xmax, ymin, zmin);
	w[2] = _vmult(&(m->m03), xmin, ymax, zmin);
	w[3] = _vmult(&(m->m03), xmax, ymax, zmin);
	w[4] = _vmult(&(m->m03), xmin, ymin, zmax);
	w[5] = _vmult(&(m->m03), xmax, ymin, zmax);
	w[6] = _vmult(&(m->m03), xmin, ymax, zmax);
	w[7] = _vmult(&(m->m03), xmax, ymax, zmax);


	// Find the 2D bounding box of the 3D bounding box
	xmin = ymin = zmin = FLT_MAX;
	xmax = ymax = zmax = -FLT_MAX;
	
	for (i=0; i<8; i++) {
		float xp = x[i];
		float yp = y[i];
		float zp = z[i];
		float wp = w[i];

		// If corner is to be clipped...	
		if (xp < -wp || 
			yp < -wp ||
			zp < -wp ||
			xp > wp ||
			yp > wp ||
			zp > wp) {


			// Point has three edges
			for (j=0; j<3; j++) {
				// Handle the clipping...
				int k = c[i][j];
				float xk = x[k];
				float yk = y[k];
				float zk = z[k];
				float wk = w[k];
/*				xp = x[i];
				yp = y[i];
				zp = z[i];
				wp = w[i];

				if (xp > wp) {
					float t = (wp - xp) / (xk-xp-wk+wp);
					if (t < 0.0f || t > 1.0f)
						continue;
					wp = wp + (wk-wp) * t;
					xp = wp;
					yp = yp + (yk-yp) * t;
					zp = zp + (zk-zp) * t;
				}
				if (yp > wp) {
					float t = (wp - yp) / (yk-yp-wk+wp);
					if (t < 0.0f || t > 1.0f)
						continue;
					wp = wp + (wk-wp) * t;
					xp = xp + (xk-xp) * t;
					yp = wp;
					zp = zp + (zk-zp) * t;
				}
				if (zp > wp) {
					float t = (wp - zp) / (zk-zp-wk+wp);
					if (t < 0.0f || t > 1.0f)
						continue;
					wp = wp + (wk-wp) * t;
					xp = xp + (xk-xp) * t;
					yp = yp + (yk-yp) * t;
					zp = wp;
				}
				if (xp < -wp) {
					float t = (wp + xp) / (xp+wp-xk-wk);
					if (t < 0.0f || t > 1.0f)
						continue;
					wp = wp + (wk-wp) * t;
					xp = -wp;
					yp = yp + (yk-yp) * t;
					zp = zp + (zk-zp) * t;
				}
				if (yp < -wp) {
					float t = (wp + yp) / (yp+wp-yk-wk);
					if (t < 0.0f || t > 1.0f)
						continue;
					wp = wp + (wk-wp) * t;
					xp = xp + (xk-xp) * t;
					yp = -wp;
					zp = zp + (zk-zp) * t;
				}*/
				if (zp < -wp) {
					float t = (wp + zp) / (zp+wp-zk-wk);
					if (t < 0.0f || t > 1.0f)
						continue;
					wp = wp + (wk-wp) * t;
					xp = xp + (xk-xp) * t;
					yp = yp + (yk-yp) * t;
					zp = -wp;
				}
				xp /= wp;
				yp /= wp;
				zp /= wp;
				if (xp < xmin) xmin = xp;
				if (xp > xmax) xmax = xp;
				if (yp < ymin) ymin = yp;
				if (yp > ymax) ymax = yp;
				if (zp < zmin) zmin = zp;
				if (zp > zmax) zmax = zp;
			}
		} else {
			// corner was not clipped..
			xp /= wp;
			yp /= wp;
			zp /= wp;
			if (xp < xmin) xmin = xp;
			if (xp > xmax) xmax = xp;
			if (yp < ymin) ymin = yp;
			if (yp > ymax) ymax = yp;
			if (zp < zmin) zmin = zp;
			if (zp > zmax) zmax = zp;
		}
	}

	/* Copy for export */
	__bktscreenbox[0]= xmin;
	__bktscreenbox[1]= xmax;
	__bktscreenbox[2]= ymin;
	__bktscreenbox[3]= ymax;
	__bktscreenbox[4]= zmin;
	__bktscreenbox[5]= zmax;

	// Now we're down to 4 points
	// (xmin, ymin)
	// (xmax, ymin)
	// (xmin, ymax)
	// (xmax, ymax)
	retval = 0;
	for (i=0; i<numprojectors; i++) {
		if (valid[i] &&
			xmin < p[i].x2 && xmax > p[i].x1 &&
			ymin < p[i].y2 && ymax > p[i].y1)
			retval |= 1 << i;
	}

	if (!g->current.beginend) {
		bkt->active = 0;
		__bktactive = 0;
	}

	*bkt->count = 0;
	if (bkt->hint != WIREGL_OBJECT_BBOX_HINT) {
		*bkt->bmin = maxvector;
		*bkt->bmax = minvector;
	}

	__retval = retval;
	return retval;
}

void
__glbucket_endflush () {
	GLcontext *g = GetCurrentContext();
	GLcurrentstate *c = &(g->current);
	GLclientstate *cl = &(g->client);
	c->beginend_orphan = c->beginend;
	c->mode_orphan = c->mode;
	c->beginendnum_orphan = c->beginendnum;
	cl->list_size = 0;
}


GLfloat *
__glbucket_getobjectbox(void) {
	return __bktobjectbox;
}

GLfloat *
__glbucket_getscreenbox(void) {
	return __bktscreenbox;
}

GLmatrix *
__glbucket_getcompositematrix(void) {
	GLcontext *g = GetCurrentContext();
	GLtransstate *t = &(g->trans);
    GLmatrix *m = &(t->transform);
	return m;
}

// Broadcast functions
void GLSTATE_DECL
__glstate_Accum (GLenum op, GLfloat value) {
	GLcontext *g = GetCurrentContext();
	GLbucketstate *bkt = &(g->bucket);

	if (IsBucketingActive())
		bkt->flush(g);

	if (g->current.beginend)
		if (__glerror(__LINE__, __FILE__, GL_INVALID_OPERATION, 
			"Accum called in begin/end"))
			return;

	if (op != GL_ACCUM &&
		op != GL_LOAD &&
		op != GL_ADD &&
		op != GL_MULT &&
		op != GL_RETURN) {
		if (__glerror(__LINE__, __FILE__, GL_INVALID_ENUM, 
			"Accum called with bogus enum: %d", op))
			return;
	}

	if (__bktnobuffer) {	
		bkt->flush(g);
		__glhw_Accum (op, value);
		return;
	}

	__glhw_Accum (op, value);
	__bktbroadcast = 1;
	bkt->flush(g);
	__bktbroadcast = 0;
}


void GLSTATE_DECL
__gltrack_Accum (GLenum op, GLfloat value) {
	__glhw_Accum (op, value);
}

void GLSTATE_DECL
__glstate_CallLists (GLsizei n, GLenum type, const GLvoid *lists) {
	GLcontext *g = GetCurrentContext();
	GLlistsstate *l = &(g->lists);
	GLbucketstate *bkt = &(g->bucket);

	if (bkt->active || __bktnobuffer)
		bkt->flush(g);

	__glhw_ListBase (l->base);
	__glhw_CallLists (n, type, lists);
	__bktbroadcast = 1;
	bkt->flush(g);
	__bktbroadcast = 0;
}

void GLSTATE_DECL
__glstate_CallList (GLuint list){ 
	GLcontext *g = GetCurrentContext();
	GLbucketstate *bkt = &(g->bucket);

	if (bkt->active || __bktnobuffer)
		bkt->flush(g);

	__glhw_CallList (list);
	__bktbroadcast = 1;
	bkt->flush(g);
	__bktbroadcast = 0;
}

void GLSTATE_DECL
__glstate_Clear (GLbitfield mask) {
	GLcontext *g = GetCurrentContext();
	GLbucketstate *bkt = &(g->bucket);
	GLbitvalue update;

	if (IsBucketingActive())
		bkt->flush(g);

	if (g->current.beginend)
		if (__glerror(__LINE__, __FILE__, GL_INVALID_OPERATION, 
			"Clear called in begin/end"))
			return;

	if (mask & 
		~GL_COLOR_BUFFER_BIT & 
		~GL_DEPTH_BUFFER_BIT &
		~GL_ACCUM_BUFFER_BIT &
		~GL_STENCIL_BUFFER_BIT)
		if (__glerror(__LINE__, __FILE__, GL_INVALID_VALUE, 
			"Clear called with bogus mask: %d", mask))
			return;

	if (__bktnobuffer) {	
		update = g->update;
		g->update = GLUPDATE_BUFFER | GLUPDATE_SCISSOR;
		bkt->flush(g);
		g->update = update;
		__bktbroadcast = 0;
		__glhw_Clear (mask);
		return;
	}

	__bktbroadcast = 1;
	update = g->update;
	g->update = GLUPDATE_BUFFER | GLUPDATE_SCISSOR;
	__glhw_Clear (mask);
	bkt->flush(g);
	g->update = update;
	__bktbroadcast = 0;
}

void GLSTATE_DECL
__gltrack_Clear (GLbitfield mask) {
	__glhw_Clear (mask);
}


void GLSTATE_DECL 
__glstate_DrawPixels (GLsizei width, GLsizei height, 
		GLenum format, GLenum type, const GLvoid *pixels)
{
	GLcontext *g = GetCurrentContext();
	GLcurrentstate *c = &(g->current);
	GLbucketstate *bkt = &(g->bucket);
	
	GLviewportstate *v = &(g->viewport);

	GLenum hint;

	if (c->beginend)
		if (__glerror(__LINE__, __FILE__, GL_INVALID_OPERATION, 
			"DrawPixels called in begin/end"))
			return;

	if (width < 0 || height < 0)
		if (__glerror(__LINE__, __FILE__, GL_INVALID_VALUE,
			"DrawPixels called with neg dims: %dx%d", width, height))
			return;

	if (!c->rastervalid)
		return;

	if (IsBucketingActive() || __bktnobuffer)
		bkt->flush(g);

	 //  Note, we need to pack first since it might cause a flush.
	__glhw_DrawPixels (width, height, format, type, pixels);

	bkt->bmin->x = (c->rasterpos.x)/v->w;
	bkt->bmin->y = (c->rasterpos.y)/v->h;
    bkt->bmax->x = (c->rasterpos.x + width)/v->w;
    bkt->bmax->y = (c->rasterpos.y + height)/v->h;

	bkt->bmin->x *= 2.0f;
	bkt->bmin->y *= 2.0f;
	bkt->bmax->x *= 2.0f;
	bkt->bmax->y *= 2.0f;

	bkt->bmin->x -= 1.0f;
	bkt->bmin->y -= 1.0f;
	bkt->bmax->x -= 1.0f;
	bkt->bmax->y -= 1.0f;

	hint = bkt->hint;
	bkt->hint = WIREGL_SCREEN_BBOX_HINT;
	bkt->flush(g);
	bkt->hint = hint;
	*bkt->bmin = maxvector;
    *bkt->bmax = minvector;
}

void GLSTATE_DECL
__glstate_Bitmap (	GLsizei width, GLsizei height,
					GLfloat xorig, GLfloat yorig,
					GLfloat xmove, GLfloat ymove,
					const GLubyte * bitmap) {
	GLcontext *g = GetCurrentContext();
	GLbucketstate *bkt = &(g->bucket);
	GLcurrentstate *c = &(g->current);
	GLviewportstate *v = &(g->viewport);
	GLstatebits *s = GetStateBits();
	GLcurrentbits *cb = &(s->current);
	GLenum hint;
	
	if (g->current.beginend)
		if (__glerror(__LINE__, __FILE__, GL_INVALID_OPERATION, 
			"Bitmap called in begin/end"))
			return;

	if (width < 0 || height < 0)
		if (__glerror(__LINE__, __FILE__, GL_INVALID_VALUE,
			"Bitmap called with neg dims: %dx%d", width, height))
			return;

	if (!c->rastervalid)
		return;

	if (IsBucketingActive() || __bktnobuffer)
		bkt->flush(g);

	 //  Note, we need to pack first since it might cause a flush.
	__glhw_Bitmap (width, height, xorig, yorig, xmove, ymove, bitmap);

	bkt->bmin->x = (c->rasterpos.x - xorig)/v->w;
	bkt->bmin->y = (c->rasterpos.y - yorig)/v->h;
    bkt->bmax->x = (c->rasterpos.x - xorig + width)/v->w;
    bkt->bmax->y = (c->rasterpos.y - yorig + height)/v->h;

	bkt->bmin->x *= 2.0f;
	bkt->bmin->y *= 2.0f;
	bkt->bmax->x *= 2.0f;
	bkt->bmax->y *= 2.0f;

	bkt->bmin->x -= 1.0f;
	bkt->bmin->y -= 1.0f;
	bkt->bmax->x -= 1.0f;
	bkt->bmax->y -= 1.0f;

	c->rasterpos.x += xmove;
	c->rasterpos.y += ymove;
	cb->raster = g->nbitID;
	cb->dirty = g->nbitID;

	hint = bkt->hint;
	bkt->hint = WIREGL_SCREEN_BBOX_HINT;
	bkt->flush(g);
	bkt->hint = hint;
	*bkt->bmin = maxvector;
    *bkt->bmax = minvector;

	c->rasterpos_pre.x += xmove;
	c->rasterpos_pre.y += ymove;
}
