/*
** glclient.h
**
** Written by Ian Buck
** 6/15/2000
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#ifndef GLCLIENT_H
#define GLCLIENT_H

#include "gltypes.h"
#include "glconfig.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
	GLbitvalue	dirty;
	GLbitvalue	enableclientstate;
	GLbitvalue	clientpointer;
	GLbitvalue  element;
	GLbitvalue  *v;
	GLbitvalue  *n;
	GLbitvalue  *c;
	GLbitvalue  *i;
	GLbitvalue  *t;
	GLbitvalue  *e;
	int valloc;
	int nalloc;
	int calloc;
	int ialloc;
	int talloc;
	int ealloc;
} GLclientbits;

typedef struct {
	unsigned char *p;
	GLint size;
	GLint type;
	GLint stride;
	GLboolean enabled;

	int bytesperindex;

	unsigned char *cache;
	int cache_alloc;
} GLclientpointer;

typedef struct {
	GLclientpointer v;
	GLclientpointer n;
	GLclientpointer c;
	GLclientpointer i;
	GLclientpointer t;
	GLclientpointer e;

	int *list;
	int list_alloc;
	int list_size;

	GLint	maxelementsindices;
	GLint	maxelementsvertices;
} GLclientstate;

void __glclient_initbits(GLclientbits *c, GLconfig *cfg);
void __glclient_switch(GLclientbits *c, GLbitvalue bitID, 
					   GLclientstate *current, GLclientstate *target);
void __glclient_sync(GLclientbits *c, GLbitvalue bitID, 
					   GLclientstate *current, GLclientstate *target);

void __glclient_init (GLclientstate *c, GLconfig *cfg);
void __glclient_destroy (GLclientstate *c);
void __glclient_setclientstate (GLclientstate *c, GLenum array, GLboolean state);
void __glclient_setcppointer (GLclientpointer *cp, GLint size, GLenum type, GLsizei stride, const GLvoid *pointer);

void __glclient_UpdateClientPointer (int array, int index_size, int *index, int data_size, unsigned char * data);
void __glclient_SendUpdates(GLclientbits *c, GLbitvalue bitID, 
							GLclientstate *current, GLclientstate *target);

void GLSTATE_DECL __glstate_LockArraysEXT (GLint first, GLsizei count);
void GLSTATE_DECL __glstate_UnlockArraysEXT (void);
void GLSTATE_DECL __glstate_EnableClientState (GLenum array);
void GLSTATE_DECL __glstate_DisableClientState (GLenum array);
void GLSTATE_DECL __glstate_VertexPointer(GLint size, GLenum type, GLsizei stride, const GLvoid *p);
void GLSTATE_DECL __glstate_ColorPointer(GLint size, GLenum type, GLsizei stride, const GLvoid *p);
void GLSTATE_DECL __glstate_IndexPointer(GLenum type, GLsizei stride, const GLvoid *p);
void GLSTATE_DECL __glstate_NormalPointer(GLenum type, GLsizei stride, const GLvoid *p);
void GLSTATE_DECL __glstate_TexCoordPointer(GLint size, GLenum type, GLsizei stride, const GLvoid *p);
void GLSTATE_DECL __glstate_EdgeFlagPointer(GLsizei stride, const GLvoid *p);
void GLSTATE_DECL __glstate_InterleavedArrays(GLenum format, GLsizei stride, const GLvoid *p);
void GLSTATE_DECL __glstate_DrawArrays(GLenum mode, GLint first, GLsizei count);
void GLSTATE_DECL __glstate_ArrayElement(GLint index);
void GLSTATE_DECL __glstate_ArrayElement_locked(GLint index);
void GLSTATE_DECL __glstate_ArrayElement_resolve (GLint index);
void GLSTATE_DECL __glstate_GetPointerv(GLenum pname, GLvoid * * params);
void GLSTATE_DECL __glstate_DrawArrays(GLenum mode, GLint first, GLsizei count);
void GLSTATE_DECL __glstate_DrawElements( GLenum mode,  GLsizei count,  GLenum type,  const GLvoid *indices);

#ifdef __cplusplus
}
#endif

#endif


