##
## glcolor.pl
##
## Written by Ian Buck
## 6/15/2000
## http://graphics.stanford.edu/software/wiregl
##
## Copyright  2000 The Board of Trustees of The Leland Stanford Junior
## University. All rights reserved.
## Except for commercial resale, lease, license or other commercial
## transactions, permission is hereby given to use, copy, and/or modify
## this software, provided that the above copyright notice and this
## permission notice appear in all copies of this software.  No part of
## this software or any derivatives thereof may be used in graphics systems
## for resale or for use in a commercial product.
##
## This software is provided "as is" and without warranty of any kind, 
## express, implied or otherwise, including without limitation, any warranty 
## of merchantability or fitness for a particular purpose.
##

sub printfunc;

print << 'EOF';
/* 
** glcolor_gen.c
**
** Generated from glcolor.pl
** Created by Ian Buck
**
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#include "glcontext.h"
#include "glerror.h"
#include "glhw.h"

EOF

$text =<< 'EOF';
print << "EEOF";
void GLSTATE_DECL
__gl${prefix}_Color3$postfix ($type red, $type green, $type blue) {
  GLcontext *g = GetCurrentContext();
  GLcurrentstate *c = &(g->current);
  GLstatebits *sb = GetStateBits();
  GLcurrentbits *cb = &(sb->current);
  
  ${\"if (IsBucketingActive())\n    g->bucket.flush(g);" if ($prefix eq "state")}

  c->color$postfix.r = red;
  c->color$postfix.g = green;
  c->color$postfix.b = blue;
  c->color$postfix.a = GL_MAX$typemax;
  c->colortype = GL_$typeenum;

  cb->dirty = g->nbitID;
  cb->color = g->nbitID;
}


void GLSTATE_DECL
__gl${prefix}_Color4$postfix ($type red, $type green, $type blue, $type alpha) {
  GLcontext *g = GetCurrentContext();
  GLcurrentstate *c = &(g->current);
  GLstatebits *sb = GetStateBits();
  GLcurrentbits *cb = &(sb->current);
  
  ${\"if (IsBucketingActive())\n    g->bucket.flush(g);" if ($prefix eq "state")}

  c->color$postfix.r = red;
  c->color$postfix.g = green;
  c->color$postfix.b = blue;
  c->color$postfix.a = alpha;
  c->colortype = GL_$typeenum;

  cb->dirty = g->nbitID;
  cb->color = g->nbitID;
}

void GLSTATE_DECL
__gl${prefix}_Color3${postfix}v (const $type *v) {
  GLcontext *g = GetCurrentContext();
  GLcurrentstate *c = &(g->current);
  GLstatebits *sb = GetStateBits();
  GLcurrentbits *cb = &(sb->current);
  
  ${\"if (IsBucketingActive())\n    g->bucket.flush(g);" if ($prefix eq "state")}

  c->color$postfix.r = v[0];
  c->color$postfix.g = v[1];
  c->color$postfix.b = v[2];
  c->color$postfix.a = GL_MAX$typemax;
  c->colortype = GL_$typeenum;

  cb->dirty = g->nbitID;
  cb->color = g->nbitID;
}


void GLSTATE_DECL
__gl${prefix}_Color4${postfix}v (const $type *v) {
  GLcontext *g = GetCurrentContext();
  GLcurrentstate *c = &(g->current);
  GLstatebits *sb = GetStateBits();
  GLcurrentbits *cb = &(sb->current);
  
  ${\"if (IsBucketingActive())\n    g->bucket.flush(g);" if ($prefix eq "state")}

  c->color$postfix.r = v[0];
  c->color$postfix.g = v[1];
  c->color$postfix.b = v[2];
  c->color$postfix.a = v[3];
  c->colortype = GL_$typeenum;

  cb->dirty = g->nbitID;
  cb->color = g->nbitID;
}
EEOF
EOF

%colors = (
	GLbyte  => {
		postfix =>  'b',
		type =>		'BYTE',
		typemax =>  'BYTE',
		},
	GLubyte  => {
		postfix =>	'ub',
		type =>		'UNSIGNED_BYTE',
		typemax =>	'UBYTE'
		},
	GLshort  => {
		postfix =>	's',
		type =>		'SHORT',
		typemax =>	'SHORT',
		},
	GLushort  => {
		postfix =>	'us',
		type =>		'UNSIGNED_SHORT',
		typemax =>	'USHORT',
		},
	GLint  => {
		postfix =>	'i',
		type =>		'INT',
		typemax =>	'INT',
		},
	GLuint  => {
		postfix =>	'ui',
		type =>		'UNSIGNED_INT',
		typemax =>	'UINT',
		},
	GLfloat  => {
		postfix =>	'f',
		type =>		'FLOAT',
		typemax =>	'FLOAT',
		},
	GLdouble  => {
		postfix =>	'd',
		type =>		'DOUBLE',
		typemax =>	'DOUBLE',
		},

);

@types = ("GLbyte", "GLubyte", "GLshort", "GLushort", "GLint", "GLuint", "GLfloat", "GLdouble");
foreach $type (@types) {
	printfunc ($type, $colors{$type}{postfix}, $colors{$type}{type}, $colors{$type}{typemax});
}

sub doflush {
$prefix = $_[0];
if ($prefix eq "state") {
	return '\n';
} 
return "";
}

sub printfunc {
$type = $_[0];
$postfix = $_[1];
$typeenum = $_[2];
$typemax = $_[3];
$prefix = "state";
eval $text;
$prefix = "track";
eval $text;
};
