/*
** glcurrent.h
**
** Written by Ian Buck
** 6/15/2000
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#ifndef GLCURRENT_H
#define GLCURRENT_H

#include "gltypes.h"
#include "glconfig.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
	GLbitvalue	dirty;
	GLbitvalue  enable;
	GLbitvalue	color;
	GLbitvalue	index;
	GLbitvalue	texcoord;
	GLbitvalue	normal;
	GLbitvalue	raster;
	GLbitvalue	edgeflag;
} GLcurrentbits; 

typedef struct {
	union {
		GLvectorb		colorb;
		GLvectorub		colorub;
		GLvectors		colors;
		GLvectorus		colorus;
		GLvectori		colori;
		GLvectorui		colorui;
		GLvectorf		colorf;
		GLvectord		colord;
	};
	GLenum			colortype;

	union {
		GLshort			indexs;
		GLint			indexi;
		GLfloat			indexf;
		GLdouble		indexd;
	};
	GLenum			indextype;
	
	union {
		GLvectors		texcoords;
		GLvectori		texcoordi;
		GLvectorf		texcoordf;
		GLvectord		texcoordd;
	};
	GLenum			texcoordtype;

	union {
		GLvectorf		normalf;
		GLvectord		normald;
	};
	GLenum			normaltype;

	GLvectorf		rasterpos;
	GLvectorf		rasterpos_pre;

	GLfloat			rasterdistance;
	GLvectorf		rastercolor;
	GLvectorf		rastertexture;
	GLdouble		rasterindex;
	GLboolean		rastervalid;

	GLboolean		edgeflag;

	GLboolean		normalize;

	GLboolean		beginend;
	GLenum			mode;
	GLint			beginendnum;

	GLboolean		beginend_orphan;
	GLenum			mode_orphan;
	GLint			beginendnum_orphan;

} GLcurrentstate; 

void __glcurrent_initbits (GLcurrentbits *, GLconfig *);
void __glcurrent_switch (GLcurrentbits *, GLbitvalue bitID,
							 GLcurrentstate *, GLcurrentstate *);
void __glcurrent_sync (GLcurrentbits *, GLbitvalue bitID,
							 GLcurrentstate *, GLcurrentstate *);

void __glcurrent_init (GLcurrentstate *c, GLconfig *cfg);
void __glcurrent_destroy (GLcurrentstate *c);


void GLSTATE_DECL __glstate_Color3b (GLbyte red, GLbyte green, GLbyte blue);
void GLSTATE_DECL __glstate_Color3bv (const GLbyte *v);
void GLSTATE_DECL __glstate_Color3d (GLdouble red, GLdouble green, GLdouble blue);
void GLSTATE_DECL __glstate_Color3dv (const GLdouble *v);
void GLSTATE_DECL __glstate_Color3f (GLfloat red, GLfloat green, GLfloat blue);
void GLSTATE_DECL __glstate_Color3fv (const GLfloat *v);
void GLSTATE_DECL __glstate_Color3i (GLint red, GLint green, GLint blue);
void GLSTATE_DECL __glstate_Color3iv (const GLint *v);
void GLSTATE_DECL __glstate_Color3s (GLshort red, GLshort green, GLshort blue);
void GLSTATE_DECL __glstate_Color3sv (const GLshort *v);
void GLSTATE_DECL __glstate_Color3ub (GLubyte red, GLubyte green, GLubyte blue);
void GLSTATE_DECL __glstate_Color3ubv (const GLubyte *v);
void GLSTATE_DECL __glstate_Color3ui (GLuint red, GLuint green, GLuint blue);
void GLSTATE_DECL __glstate_Color3uiv (const GLuint *v);
void GLSTATE_DECL __glstate_Color3us (GLushort red, GLushort green, GLushort blue);
void GLSTATE_DECL __glstate_Color3usv (const GLushort *v);
void GLSTATE_DECL __glstate_Color4b (GLbyte red, GLbyte green, GLbyte blue, GLbyte alpha);
void GLSTATE_DECL __glstate_Color4bv (const GLbyte *v);
void GLSTATE_DECL __glstate_Color4d (GLdouble red, GLdouble green, GLdouble blue, GLdouble alpha);
void GLSTATE_DECL __glstate_Color4dv (const GLdouble *v);
void GLSTATE_DECL __glstate_Color4f (GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha);
void GLSTATE_DECL __glstate_Color4fv (const GLfloat *v);
void GLSTATE_DECL __glstate_Color4i (GLint red, GLint green, GLint blue, GLint alpha);
void GLSTATE_DECL __glstate_Color4iv (const GLint *v);
void GLSTATE_DECL __glstate_Color4s (GLshort red, GLshort green, GLshort blue, GLshort alpha);
void GLSTATE_DECL __glstate_Color4sv (const GLshort *v);
void GLSTATE_DECL __glstate_Color4ub (GLubyte red, GLubyte green, GLubyte blue, GLubyte alpha);
void GLSTATE_DECL __glstate_Color4ubv (const GLubyte *v);
void GLSTATE_DECL __glstate_Color4ui (GLuint red, GLuint green, GLuint blue, GLuint alpha);
void GLSTATE_DECL __glstate_Color4uiv (const GLuint *v);
void GLSTATE_DECL __glstate_Color4us (GLushort red, GLushort green, GLushort blue, GLushort alpha);
void GLSTATE_DECL __glstate_Color4usv (const GLushort *v);

void GLSTATE_DECL __gltrack_Color3b (GLbyte red, GLbyte green, GLbyte blue);
void GLSTATE_DECL __gltrack_Color3bv (const GLbyte *v);
void GLSTATE_DECL __gltrack_Color3d (GLdouble red, GLdouble green, GLdouble blue);
void GLSTATE_DECL __gltrack_Color3dv (const GLdouble *v);
void GLSTATE_DECL __gltrack_Color3f (GLfloat red, GLfloat green, GLfloat blue);
void GLSTATE_DECL __gltrack_Color3fv (const GLfloat *v);
void GLSTATE_DECL __gltrack_Color3i (GLint red, GLint green, GLint blue);
void GLSTATE_DECL __gltrack_Color3iv (const GLint *v);
void GLSTATE_DECL __gltrack_Color3s (GLshort red, GLshort green, GLshort blue);
void GLSTATE_DECL __gltrack_Color3sv (const GLshort *v);
void GLSTATE_DECL __gltrack_Color3ub (GLubyte red, GLubyte green, GLubyte blue);
void GLSTATE_DECL __gltrack_Color3ubv (const GLubyte *v);
void GLSTATE_DECL __gltrack_Color3ui (GLuint red, GLuint green, GLuint blue);
void GLSTATE_DECL __gltrack_Color3uiv (const GLuint *v);
void GLSTATE_DECL __gltrack_Color3us (GLushort red, GLushort green, GLushort blue);
void GLSTATE_DECL __gltrack_Color3usv (const GLushort *v);
void GLSTATE_DECL __gltrack_Color4b (GLbyte red, GLbyte green, GLbyte blue, GLbyte alpha);
void GLSTATE_DECL __gltrack_Color4bv (const GLbyte *v);
void GLSTATE_DECL __gltrack_Color4d (GLdouble red, GLdouble green, GLdouble blue, GLdouble alpha);
void GLSTATE_DECL __gltrack_Color4dv (const GLdouble *v);
void GLSTATE_DECL __gltrack_Color4f (GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha);
void GLSTATE_DECL __gltrack_Color4fv (const GLfloat *v);
void GLSTATE_DECL __gltrack_Color4i (GLint red, GLint green, GLint blue, GLint alpha);
void GLSTATE_DECL __gltrack_Color4iv (const GLint *v);
void GLSTATE_DECL __gltrack_Color4s (GLshort red, GLshort green, GLshort blue, GLshort alpha);
void GLSTATE_DECL __gltrack_Color4sv (const GLshort *v);
void GLSTATE_DECL __gltrack_Color4ub (GLubyte red, GLubyte green, GLubyte blue, GLubyte alpha);
void GLSTATE_DECL __gltrack_Color4ubv (const GLubyte *v);
void GLSTATE_DECL __gltrack_Color4ui (GLuint red, GLuint green, GLuint blue, GLuint alpha);
void GLSTATE_DECL __gltrack_Color4uiv (const GLuint *v);
void GLSTATE_DECL __gltrack_Color4us (GLushort red, GLushort green, GLushort blue, GLushort alpha);
void GLSTATE_DECL __gltrack_Color4usv (const GLushort *v);

void GLSTATE_DECL __glstate_RasterPos2d(GLdouble x, GLdouble y);
void GLSTATE_DECL __glstate_RasterPos2f(GLfloat x, GLfloat y);
void GLSTATE_DECL __glstate_RasterPos2i(GLint x, GLint y);
void GLSTATE_DECL __glstate_RasterPos2s(GLshort x, GLshort y);
void GLSTATE_DECL __glstate_RasterPos3d(GLdouble x, GLdouble y, GLdouble z);
void GLSTATE_DECL __glstate_RasterPos3f(GLfloat x, GLfloat y, GLfloat z);
void GLSTATE_DECL __glstate_RasterPos3i(GLint x, GLint y, GLint z);
void GLSTATE_DECL __glstate_RasterPos3s(GLshort x, GLshort y, GLshort z);
void GLSTATE_DECL __glstate_RasterPos4d(GLdouble x, GLdouble y, GLdouble z, GLdouble w);
void GLSTATE_DECL __glstate_RasterPos4f(GLfloat x, GLfloat y, GLfloat z, GLfloat w);
void GLSTATE_DECL __glstate_RasterPos4i(GLint x, GLint y, GLint z, GLint w);
void GLSTATE_DECL __glstate_RasterPos4s(GLshort x, GLshort y, GLshort z, GLshort w);
void GLSTATE_DECL __glstate_RasterPos2dv(const GLdouble *v);
void GLSTATE_DECL __glstate_RasterPos2fv(const GLfloat *v);
void GLSTATE_DECL __glstate_RasterPos2iv(const GLint *v);
void GLSTATE_DECL __glstate_RasterPos2sv(const GLshort *v);
void GLSTATE_DECL __glstate_RasterPos3dv(const GLdouble *v);
void GLSTATE_DECL __glstate_RasterPos3fv(const GLfloat *v);
void GLSTATE_DECL __glstate_RasterPos3iv(const GLint *v);
void GLSTATE_DECL __glstate_RasterPos3sv(const GLshort *v);
void GLSTATE_DECL __glstate_RasterPos4dv(const GLdouble *v);
void GLSTATE_DECL __glstate_RasterPos4fv(const GLfloat *v);
void GLSTATE_DECL __glstate_RasterPos4iv(const GLint *v);
void GLSTATE_DECL __glstate_RasterPos4sv(const GLshort *v);

void GLSTATE_DECL __glstate_Begin (GLenum mode);
void GLSTATE_DECL __glstate_End (void);

void GLSTATE_DECL __gltrack_Begin (GLenum mode);
void GLSTATE_DECL __gltrack_End (void);

#ifdef __cplusplus
}
#endif

#endif
