/*
** wiregl_client.h
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#ifndef WIREGL_CLIENT_H
#define WIREGL_CLIENT_H

#include "wiregl/include/wiregl_util.h"
#include "wiregl/glcontext/glcontext.h"
#include "wiregl/glcontext/glhw.h"
#include "wiregl/include/wiregl_bbox.h"

#ifdef WINDOWS
#include <windows.h>
#endif

extern unsigned char *pipe_server_opcode_start, *pipe_server_opcode_current;
extern unsigned char *pipe_server_data_start, *pipe_server_data_current, *pipe_server_data_end;
extern unsigned char *pipe_server_opcode_start_one, *pipe_server_opcode_start_two;
extern unsigned char *pipe_server_data_start_one, *pipe_server_data_start_two;
extern unsigned int pipe_server_buffer_size;
extern GLcontext *pipe_server_state;

typedef struct {
	WireGLConnection *connection;
	int w,h;
	int mural_x, mural_y;
	int window_x, window_y;
	int feather_top_size;
	int feather_bottom_size;
	int feather_left_size;
	int feather_right_size;
	float feather_top[4];
	float feather_bottom[4];
	float feather_left[4];
	float feather_right[4];
	int feather_top_dice;
	int feather_bottom_dice;
	int feather_left_dice;
	int feather_right_dice;

	GLcontext *context;

	char *server;
	int index;

	unsigned char *data_start, *data_current, *data_end;
	unsigned char *opcode_start, *opcode_current;
} WireGLPipeServer;

#ifdef WINDOWS
extern HDC client_hdc;
extern HWND client_hwnd;
#endif

void wireGLClientParseConfig( char *filename );

extern int num_pipe_servers;
extern WireGLPipeServer *pipe_servers;

extern int print_bytes_per_server;
extern int print_bytes_per_frame_per_server;
extern int print_min_bytes_sent;
extern int print_frame_rate;
extern int print_flush_time;
extern int print_send_time;
extern int print_diff_context_time;
extern int print_total_bytes_sent;
extern int print_total_time;

extern int wiregl_mural_width, wiregl_mural_height;

extern GLjumptable wiregl_im_api[];
extern GLapi wiregl_pack_api[];

void wireGLSwapBuffers();
void wireGLCreateContext();

#endif /* WIREGL_CLIENT_H */

