#ifndef WIREGL_PIPE_H
#define WIREGL_PIPE_H

#ifdef WINDOWS
#include <windows.h>
#endif

#include "wiregl/include/wiregl_util.h"
#include "wiregl/include/wiregl_protocol.h"

#ifndef NO_INCLUDE_GLCONTEXT_STUFF
#include "wiregl/glcontext/glcontext.h"
#else
struct GLcontext;
#endif

#if defined( IRIX ) || defined( IRIX64 ) || defined( Linux)
#include <X11/Xlib.h>
#include <X11/Xutil.h>
typedef void *GLXContext;
#endif


typedef struct {
	WireGLConnection *conn;
	char *fname;
	int port;
	int window_width, window_height;
	int window_x, window_y;
	int feather_top_size;
	int feather_bottom_size;
	int feather_left_size;
	int feather_right_size;
	float feather_top[4];
	float feather_bottom[4];
	float feather_left[4];
	float feather_right[4];
	int feather_top_dice;
	int feather_bottom_dice;
	int feather_left_dice;
	int feather_right_dice;

	GLcontext *state;

	WireGLConnection *barrier_conn;
	int swap_barrier;
#ifdef WINDOWS
	HWND hWnd;
	HGLRC hRC;
	HDC device_context;
#else
	Display *dpy;
	XVisualInfo *vi;
	GLXContext cx;
	Window win;
#endif
} WireGLPipe;

extern WireGLPipe wiregl_pipe;

void wireGLReadManyPackets( WireGLConnection *conn, int *num_opcodes, unsigned char **opcodes, char **data );

void wireGLWaitForConnection( );
void wireGLCreateWindow( void );
void wireGLRenderRemoteStream( void );

void wireGLPipeParseConfig( char *fname );

typedef void (*wiregl_decoder)(void);
extern wiregl_decoder wiregl_decode_functions[];
extern char *unpacker_data_ptr;

#define READ_DATA( offset, type ) \
	*( (type *) (unpacker_data_ptr + (offset)))

#define DATA_POINTER( offset, type ) \
	( (type *) (unpacker_data_ptr + (offset)) )

#define INCR_DATA_PTR( delta ) \
	unpacker_data_ptr += (delta)

#define INCR_VAR_PTR() \
	INCR_DATA_PTR( *((int *) unpacker_data_ptr ) )

#endif /* WIREGL_PIPE_H */
