/*
** drawpixels.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "wiregl/include/wireGL.h"
#include "wiregl/include/wiregl_protocol.h"
#include "wiregl/include/wiregl_client.h"
#include "wiregl/include/wiregl_util.h"
#include "wiregl/glcontext/glcontext.h"
#include "packer_internals.h"

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( DrawPixels )(GLsizei width, GLsizei height, 
		GLenum format, GLenum type, const GLvoid *pixels)
{
	unsigned char *data_ptr;
	int packet_length;
	int datasize;

	if (!pixels) return;

	if (type == GL_BITMAP)
	{
		WIREGL_PACK_FUNCTION( Bitmap  )( width, height, 0, 0, 0, 0, (const GLubyte *) pixels );
		return;
	}

	packet_length = 
		sizeof( int ) + 
		sizeof( width ) + 
		sizeof( height ) +
		sizeof( format ) +
		sizeof( type );

	if (pixels)
	{
		datasize = __glpixel_getdata2Dsize(width, height, format, type );
		packet_length += datasize;
	}

	data_ptr = (unsigned char *) wireGLAllocPacket( packet_length );
	WRITE_DATA( 0, int, packet_length );
	WRITE_DATA( sizeof( int ) + 0, GLsizei, width );
	WRITE_DATA( sizeof( int ) + 4, GLsizei, height );
	WRITE_DATA( sizeof( int ) + 8, GLenum, format );
	WRITE_DATA( sizeof( int ) + 12, GLenum, type );

	if (pixels)
	{
		__glpixel_getdata2D((GLvoid *)(data_ptr + sizeof( int ) + 16),
			width, height, format, type, pixels );
	}

	wireGLSendPacket( WIREGL_DRAWPIXELS_OPCODE, data_ptr );
	wireGLFreePacket( data_ptr );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( Bitmap  )(GLsizei width, GLsizei height, 
		GLfloat xorig, GLfloat yorig, GLfloat xmove, GLfloat ymove, const GLubyte *bitmap)
{
	unsigned char *data_ptr;
	int packet_length = sizeof( int ) + sizeof( width ) + sizeof( height ) +
		sizeof( xorig ) + sizeof( yorig ) + sizeof( xmove ) + sizeof( ymove );

	int row_length = ((width+7)&~7)*sizeof( *bitmap )/8;
	int data_length = row_length*height;
	packet_length += data_length;

	GLcontext *ctx = pipe_server_state;

	if (bitmap) { 
		if (ctx->pixel.unpackrowlength != 0 )
			wireGLSimpleError( "I don't really know how to unpack this garbage yet!" );
		if (ctx->pixel.unpackalignment != 1 )
			wireGLSimpleError( "I don't really know how to unpack this garbage yet!" );
		if (ctx->pixel.unpackskiprows != 0 )
			wireGLSimpleError( "I don't really know how to unpack this garbage yet!" );
		if (ctx->pixel.unpackskippixels != 0 )
			wireGLSimpleError( "I don't really know how to unpack this garbage yet!" );
	}


	data_ptr = (unsigned char *) wireGLAllocPacket( packet_length );
	WRITE_DATA( 0, int, packet_length );
	WRITE_DATA( sizeof( int ) + 0, GLsizei, width );
	WRITE_DATA( sizeof( int ) + 4, GLsizei, height );
	WRITE_DATA( sizeof( int ) + 8, GLfloat, xorig );
	WRITE_DATA( sizeof( int ) + 12, GLfloat, yorig );
	WRITE_DATA( sizeof( int ) + 16, GLfloat, xmove );
	WRITE_DATA( sizeof( int ) + 20, GLfloat, ymove );
	if (bitmap)
		memcpy( data_ptr + sizeof( int ) + 24, bitmap, data_length );

	wireGLSendPacket( WIREGL_BITMAP_OPCODE, data_ptr );
	wireGLFreePacket( data_ptr );
}
