/*
** vertexBB.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#include "wiregl/include/wireGL.h"
#include "wiregl/include/wiregl_protocol.h"
#include "wiregl/include/wiregl_client.h"
#include "wiregl/include/wiregl_util.h"
#include "wiregl/glcontext/glcontext.h"

GLvectorf bounds_min;
GLvectorf bounds_max;
GLint vertex_count = 0;
GLint vertex_max = GL_MAXINT;

#define GET_BUFFERED_COUNT_POINTER( len ) \
  data_ptr = pipe_server_data_current; \
  if (data_ptr + (len) > pipe_server_data_end || vertex_count >= vertex_max)\
  { \
    wireGLFlushBuffer( pipe_server_state ); \
	data_ptr = pipe_server_data_current; \
  } \
  pipe_server_data_current += (len); \
  vertex_count++

#ifdef WINDOWS
#define UPDATE_2D_BBOX() \
__asm {\
	__asm fld fx\
	__asm fld bounds_min.x\
	__asm fcomi st, st(1)\
	__asm fcmovnb st(0), st(1)\
	__asm fstp bounds_min.x\
	__asm fld bounds_max.x\
	__asm fcomi st, st(1)\
	__asm fcmovb st(0), st(1)\
	__asm fstp bounds_max.x\
	__asm fstp fx\
	\
	__asm fld fy\
	__asm fld bounds_min.y\
	__asm fcomi st, st(1)\
	__asm fcmovnb st(0), st(1)\
	__asm fstp bounds_min.y\
	__asm fld bounds_max.y\
	__asm fcomi st, st(1)\
	__asm fcmovb st(0), st(1)\
	__asm fstp bounds_max.y\
	__asm fstp fy\
} \
	if (bounds_min.z > 0.0f)  bounds_min.z = 0.0f;\
	if (bounds_max.z < 0.0f)  bounds_max.z = 0.0f
#else
#define UPDATE_2D_BBOX() \
if (bounds_min.x > fx)    bounds_min.x = fx;\
if (bounds_min.y > fy)    bounds_min.y = fy;\
if (bounds_min.z > 0.0f)  bounds_min.z = 0.0f;\
if (bounds_max.x < fx)    bounds_max.x = fx;\
if (bounds_max.y < fy)    bounds_max.y = fy;\
if (bounds_max.z < 0.0f)  bounds_max.z = 0.0f
#endif

#ifdef WINDOWS
#define UPDATE_3D_BBOX() \
__asm {\
	__asm fld fx\
	__asm fld bounds_min.x\
	__asm fcomi st, st(1)\
	__asm fcmovnb st(0), st(1)\
	__asm fstp bounds_min.x\
	__asm fld bounds_max.x\
	__asm fcomi st, st(1)\
	__asm fcmovb st(0), st(1)\
	__asm fstp bounds_max.x\
	__asm fstp fx\
	\
	__asm fld fy\
	__asm fld bounds_min.y\
	__asm fcomi st, st(1)\
	__asm fcmovnb st(0), st(1)\
	__asm fstp bounds_min.y\
	__asm fld bounds_max.y\
	__asm fcomi st, st(1)\
	__asm fcmovb st(0), st(1)\
	__asm fstp bounds_max.y\
	__asm fstp fy\
	\
	__asm fld fz\
	__asm fld bounds_min.z\
	__asm fcomi st, st(1)\
	__asm fcmovnb st(0), st(1)\
	__asm fstp bounds_min.z\
	__asm fld bounds_max.z\
	__asm fcomi st, st(1)\
	__asm fcmovb st(0), st(1)\
	__asm fstp bounds_max.z\
	__asm fstp fz\
	\
}
#else
#define UPDATE_3D_BBOX() \
if (bounds_min.x > fx)    bounds_min.x = fx; \
if (bounds_min.y > fy)    bounds_min.y = fy; \
if (bounds_min.z > fz)    bounds_min.z = fz; \
if (bounds_max.x < fx)    bounds_max.x = fx; \
if (bounds_max.y < fy)    bounds_max.y = fy; \
if (bounds_max.z < fz)    bounds_max.z = fz
#endif


#define CREATE_2D_FLOATS() \
	GLfloat fx = (GLfloat) x; \
	GLfloat fy = (GLfloat) y

#define CREATE_3D_FLOATS() \
	GLfloat fx = (GLfloat) x; \
	GLfloat fy = (GLfloat) y; \
	GLfloat fz = (GLfloat) z

#define CREATE_4D_FLOATS() \
	GLfloat fx = (GLfloat) x; \
	GLfloat fy = (GLfloat) y; \
	GLfloat fz = (GLfloat) z; \
	GLfloat fw = (GLfloat) w; \
	fx /= fw; fy /= fw; fz/= fw

#define CREATE_2D_VFLOATS() \
	GLfloat fx = (GLfloat) v[0]; \
	GLfloat fy = (GLfloat) v[1]

#define CREATE_3D_VFLOATS() \
	GLfloat fx = (GLfloat) v[0]; \
	GLfloat fy = (GLfloat) v[1]; \
	GLfloat fz = (GLfloat) v[2]

#define CREATE_4D_VFLOATS() \
	GLfloat fx = (GLfloat) v[0]; \
	GLfloat fy = (GLfloat) v[1]; \
	GLfloat fz = (GLfloat) v[2]; \
	GLfloat fw = (GLfloat) v[3]; \
	fx /= fw; fy /= fw; fz/= fw


void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB2d ) ( GLdouble x, GLdouble y )
{
	unsigned char *data_ptr;
	CREATE_2D_FLOATS();
	GET_BUFFERED_COUNT_POINTER( 16 );
	UPDATE_2D_BBOX();
	WRITE_DATA( 0, GLdouble, x );
	WRITE_DATA( 8, GLdouble, y );
	WRITE_OPCODE( WIREGL_VERTEX2D_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB2dv ) ( const GLdouble *v )
{
	unsigned char *data_ptr;
	CREATE_2D_VFLOATS();
	GET_BUFFERED_COUNT_POINTER( 16 );
	UPDATE_2D_BBOX();
	WRITE_DATA( 0, GLdouble, v[0] );
	WRITE_DATA( 8, GLdouble, v[1] );
	WRITE_OPCODE( WIREGL_VERTEX2D_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB2f ) ( GLfloat fx, GLfloat fy )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 8 );
	UPDATE_2D_BBOX();
	WRITE_DATA( 0, GLfloat, fx );
	WRITE_DATA( 4, GLfloat, fy );
	WRITE_OPCODE( WIREGL_VERTEX2F_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB2fv ) ( const GLfloat *v )
{
	unsigned char *data_ptr;
	CREATE_2D_VFLOATS();
	GET_BUFFERED_COUNT_POINTER( 8 );
	UPDATE_2D_BBOX();
	WRITE_DATA( 0, GLfloat, fx );
	WRITE_DATA( 4, GLfloat, fy );
	WRITE_OPCODE( WIREGL_VERTEX2F_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB2i ) ( GLint x, GLint y )
{
	unsigned char *data_ptr;
	CREATE_2D_FLOATS();
	GET_BUFFERED_COUNT_POINTER( 8 );
	UPDATE_2D_BBOX();
	WRITE_DATA( 0, GLint, x );
	WRITE_DATA( 4, GLint, y );
	WRITE_OPCODE( WIREGL_VERTEX2I_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB2iv ) ( const GLint *v )
{
	unsigned char *data_ptr;
	CREATE_2D_VFLOATS();
	GET_BUFFERED_COUNT_POINTER( 8 );
	UPDATE_2D_BBOX();
	WRITE_DATA( 0, GLint, v[0] );
	WRITE_DATA( 4, GLint, v[1] );
	WRITE_OPCODE( WIREGL_VERTEX2I_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB2s ) ( GLshort x, GLshort y )
{
	unsigned char *data_ptr;
	CREATE_2D_FLOATS();
	GET_BUFFERED_COUNT_POINTER( 4 );
	UPDATE_2D_BBOX();
	WRITE_DATA( 0, GLshort, x );
	WRITE_DATA( 2, GLshort, y );
	WRITE_OPCODE( WIREGL_VERTEX2S_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB2sv ) ( const GLshort *v )
{
	unsigned char *data_ptr;
	CREATE_2D_VFLOATS();
	GET_BUFFERED_COUNT_POINTER( 4 );
	UPDATE_2D_BBOX();
	WRITE_DATA( 0, GLshort, v[0] );
	WRITE_DATA( 2, GLshort, v[1] );
	WRITE_OPCODE( WIREGL_VERTEX2S_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB3d ) ( GLdouble x, GLdouble y, GLdouble z )
{
	unsigned char *data_ptr;
	CREATE_3D_FLOATS();
	GET_BUFFERED_COUNT_POINTER( 24 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLdouble, x );
	WRITE_DATA( 8, GLdouble, y );
	WRITE_DATA( 16, GLdouble, z );
	WRITE_OPCODE( WIREGL_VERTEX3D_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB3dv ) ( const GLdouble *v )
{
	unsigned char *data_ptr;
	CREATE_3D_VFLOATS();
	GET_BUFFERED_COUNT_POINTER( 24 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLdouble, v[0] );
	WRITE_DATA( 8, GLdouble, v[1] );
	WRITE_DATA( 16, GLdouble, v[2] );
	WRITE_OPCODE( WIREGL_VERTEX3D_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB3f ) ( GLfloat fx, GLfloat fy, GLfloat fz )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 12 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLfloat, fx );
	WRITE_DATA( 4, GLfloat, fy );
	WRITE_DATA( 8, GLfloat, fz );
	WRITE_OPCODE( WIREGL_VERTEX3F_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB3fv ) ( const GLfloat *v )
{
	unsigned char *data_ptr;
	CREATE_3D_VFLOATS();
	GET_BUFFERED_COUNT_POINTER( 12 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLfloat, fx );
	WRITE_DATA( 4, GLfloat, fy );
	WRITE_DATA( 8, GLfloat, fz );
	WRITE_OPCODE( WIREGL_VERTEX3F_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB3i ) ( GLint x, GLint y, GLint z )
{
	unsigned char *data_ptr;
	CREATE_3D_FLOATS();
	GET_BUFFERED_COUNT_POINTER( 12 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLint, x );
	WRITE_DATA( 4, GLint, y );
	WRITE_DATA( 8, GLint, z );
	WRITE_OPCODE( WIREGL_VERTEX3I_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB3iv ) ( const GLint *v )
{
	unsigned char *data_ptr;
	CREATE_3D_VFLOATS();
	GET_BUFFERED_COUNT_POINTER( 12 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLint, v[0] );
	WRITE_DATA( 4, GLint, v[1] );
	WRITE_DATA( 8, GLint, v[2] );
	WRITE_OPCODE( WIREGL_VERTEX3I_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB3s ) ( GLshort x, GLshort y, GLshort z )
{
	unsigned char *data_ptr;
	CREATE_3D_FLOATS();
	GET_BUFFERED_COUNT_POINTER( 6 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLshort, x );
	WRITE_DATA( 2, GLshort, y );
	WRITE_DATA( 4, GLshort, z );
	WRITE_OPCODE( WIREGL_VERTEX3S_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB3sv ) ( const GLshort *v )
{
	unsigned char *data_ptr;
	CREATE_3D_VFLOATS();
	GET_BUFFERED_COUNT_POINTER( 6 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLshort, v[0] );
	WRITE_DATA( 2, GLshort, v[1] );
	WRITE_DATA( 4, GLshort, v[2] );
	WRITE_OPCODE( WIREGL_VERTEX3S_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB4d ) ( GLdouble x, GLdouble y, GLdouble z, GLdouble w )
{
	unsigned char *data_ptr;
	CREATE_4D_FLOATS();
	GET_BUFFERED_COUNT_POINTER( 32 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLdouble, x );
	WRITE_DATA( 8, GLdouble, y );
	WRITE_DATA( 16, GLdouble, z );
	WRITE_DATA( 24, GLdouble, w );
	WRITE_OPCODE( WIREGL_VERTEX4D_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB4dv ) ( const GLdouble *v )
{
	unsigned char *data_ptr;
	CREATE_4D_VFLOATS();
	GET_BUFFERED_COUNT_POINTER( 32 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLdouble, v[0] );
	WRITE_DATA( 8, GLdouble, v[1] );
	WRITE_DATA( 16, GLdouble, v[2] );
	WRITE_DATA( 24, GLdouble, v[3] );
	WRITE_OPCODE( WIREGL_VERTEX4D_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB4f ) ( GLfloat fx, GLfloat fy, GLfloat fz, GLfloat fw )
{
	unsigned char *data_ptr;
	GET_BUFFERED_COUNT_POINTER( 16 );
	fx /= fw; fy /= fw; fz /= fw;
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLfloat, fx );
	WRITE_DATA( 4, GLfloat, fy );
	WRITE_DATA( 8, GLfloat, fz );
	WRITE_DATA( 12, GLfloat, 1.0f );
	WRITE_OPCODE( WIREGL_VERTEX4F_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB4fv ) ( const GLfloat *v )
{
	unsigned char *data_ptr;
	CREATE_4D_VFLOATS();
	GET_BUFFERED_COUNT_POINTER( 16 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLfloat, v[0] );
	WRITE_DATA( 4, GLfloat, v[1] );
	WRITE_DATA( 8, GLfloat, v[2] );
	WRITE_DATA( 12, GLfloat, v[3] );
	WRITE_OPCODE( WIREGL_VERTEX4F_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB4i ) ( GLint x, GLint y, GLint z, GLint w )
{
	unsigned char *data_ptr;
	CREATE_4D_FLOATS();
	GET_BUFFERED_COUNT_POINTER( 16 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLint, x );
	WRITE_DATA( 4, GLint, y );
	WRITE_DATA( 8, GLint, z );
	WRITE_DATA( 12, GLint, w );
	WRITE_OPCODE( WIREGL_VERTEX4I_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB4iv ) ( const GLint *v )
{
	unsigned char *data_ptr;
	CREATE_4D_VFLOATS();
	GET_BUFFERED_COUNT_POINTER( 16 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLint, v[0] );
	WRITE_DATA( 4, GLint, v[1] );
	WRITE_DATA( 8, GLint, v[2] );
	WRITE_DATA( 12, GLint, v[3] );
	WRITE_OPCODE( WIREGL_VERTEX4I_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB4s ) ( GLshort x, GLshort y, GLshort z, GLshort w )
{
	unsigned char *data_ptr;
	CREATE_4D_FLOATS();
	GET_BUFFERED_COUNT_POINTER( 8 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLshort, x );
	WRITE_DATA( 2, GLshort, y );
	WRITE_DATA( 4, GLshort, z );
	WRITE_DATA( 6, GLshort, w );
	WRITE_OPCODE( WIREGL_VERTEX4S_OPCODE );
}

void OPENGL_APIENTRY WIREGL_PACK_FUNCTION( VertexBB4sv ) ( const GLshort *v )
{
	unsigned char *data_ptr;
	CREATE_4D_VFLOATS();
	GET_BUFFERED_COUNT_POINTER( 8 );
	UPDATE_3D_BBOX();
	WRITE_DATA( 0, GLshort, v[0] );
	WRITE_DATA( 2, GLshort, v[1] );
	WRITE_DATA( 4, GLshort, v[2] );
	WRITE_DATA( 6, GLshort, v[3] );
	WRITE_OPCODE( WIREGL_VERTEX4S_OPCODE );
}

GLapi vertexBBapi[] = {
	{"glVertex2d",  WIREGL_PACK_FUNCTION( VertexBB2d )},
	{"glVertex2dv", WIREGL_PACK_FUNCTION( VertexBB2dv )},
	{"glVertex2f",  WIREGL_PACK_FUNCTION( VertexBB2f )},
	{"glVertex2fv", WIREGL_PACK_FUNCTION( VertexBB2fv )},
	{"glVertex2i",  WIREGL_PACK_FUNCTION( VertexBB2i )},
	{"glVertex2iv", WIREGL_PACK_FUNCTION( VertexBB2iv )},
	{"glVertex2s",  WIREGL_PACK_FUNCTION( VertexBB2s )},
	{"glVertex2sv", WIREGL_PACK_FUNCTION( VertexBB2sv )},
	{"glVertex3d",  WIREGL_PACK_FUNCTION( VertexBB3d )},
	{"glVertex3dv", WIREGL_PACK_FUNCTION( VertexBB3dv )},
	{"glVertex3f",  WIREGL_PACK_FUNCTION( VertexBB3f )},
	{"glVertex3fv", WIREGL_PACK_FUNCTION( VertexBB3fv )},
	{"glVertex3i",  WIREGL_PACK_FUNCTION( VertexBB3i )},
	{"glVertex3iv", WIREGL_PACK_FUNCTION( VertexBB3iv )},
	{"glVertex3s",  WIREGL_PACK_FUNCTION( VertexBB3s )},
	{"glVertex3sv", WIREGL_PACK_FUNCTION( VertexBB3sv )},
	{"glVertex4d",  WIREGL_PACK_FUNCTION( VertexBB4d )},
	{"glVertex4dv", WIREGL_PACK_FUNCTION( VertexBB4dv )},
	{"glVertex4f",  WIREGL_PACK_FUNCTION( VertexBB4f )},
	{"glVertex4fv", WIREGL_PACK_FUNCTION( VertexBB4fv )},
	{"glVertex4i",  WIREGL_PACK_FUNCTION( VertexBB4i )},
	{"glVertex4iv", WIREGL_PACK_FUNCTION( VertexBB4iv )},
	{"glVertex4s",  WIREGL_PACK_FUNCTION( VertexBB4s )},
	{"glVertex4sv", WIREGL_PACK_FUNCTION( VertexBB4sv )},
	{"", NULL}};

