/*
** texture.c
** http://graphics.stanford.edu/software/wiregl
**
** Copyright  2000 The Board of Trustees of The Leland Stanford Junior
** University. All rights reserved.
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or modify
** this software, provided that the above copyright notice and this
** permission notice appear in all copies of this software.  No part of
** this software or any derivatives thereof may be used in graphics systems
** for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind, 
** express, implied or otherwise, including without limitation, any warranty 
** of merchantability or fitness for a particular purpose.
**
*/

#ifdef WINDOWS
#include <windows.h>
#endif
#include <GL/gl.h>

#include "wiregl/include/wireGL.h"
#include "wiregl/include/wiregl_protocol.h"
#include "wiregl/include/wiregl_pipe.h"
#include "wiregl/include/wiregl_util.h"

void __decodeTexImage2D( void )
{
	int is_null = READ_DATA( sizeof( int ) + 32, int );

	glPixelStorei( GL_UNPACK_ROW_LENGTH, 0 );
	glPixelStorei( GL_UNPACK_SKIP_PIXELS, 0 );
	glPixelStorei( GL_UNPACK_SKIP_ROWS, 0 );
	glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
	
	if (is_null)
	{
		glTexImage2D(
			READ_DATA( sizeof( int ) + 0, GLenum ),	/* target */
			READ_DATA( sizeof( int ) + 4, GLint ),	/* level */
			READ_DATA( sizeof( int ) + 8, GLint ),	/* internalformat */
			READ_DATA( sizeof( int ) + 12, GLsizei ),	/* width */
			READ_DATA( sizeof( int ) + 16, GLsizei ),	/* height */
			READ_DATA( sizeof( int ) + 20, GLint ),	/* border */
			READ_DATA( sizeof( int ) + 24, GLenum ),	/* format */
			READ_DATA( sizeof( int ) + 28, GLenum ),	/* type */
			NULL
			 );
	}
	else
	{
		glTexImage2D(
			READ_DATA( sizeof( int ) + 0, GLenum ),	/* target */
			READ_DATA( sizeof( int ) + 4, GLint ),	/* level */
			READ_DATA( sizeof( int ) + 8, GLint ),	/* internalformat */
			READ_DATA( sizeof( int ) + 12, GLsizei ),	/* width */
			READ_DATA( sizeof( int ) + 16, GLsizei ),	/* height */
			READ_DATA( sizeof( int ) + 20, GLint ),	/* border */
			READ_DATA( sizeof( int ) + 24, GLenum ),	/* format */
			READ_DATA( sizeof( int ) + 28, GLenum ),	/* type */
			DATA_POINTER( sizeof( int ) + 36, GLvoid )
			 );
	}
	INCR_VAR_PTR();
}

void __decodeTexImage1D( void )
{
	int is_null = READ_DATA( sizeof( int ) + 28, int );

	glPixelStorei( GL_UNPACK_ROW_LENGTH, 0 );
	glPixelStorei( GL_UNPACK_SKIP_PIXELS, 0 );
	glPixelStorei( GL_UNPACK_SKIP_ROWS, 0 );
	glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
	
	if (is_null)
	{
		glTexImage1D(
			READ_DATA( sizeof( int ) + 0, GLenum ),	/* target */
			READ_DATA( sizeof( int ) + 4, GLint ),	/* level */
			READ_DATA( sizeof( int ) + 8, GLint ),	/* internalformat */
			READ_DATA( sizeof( int ) + 12, GLsizei ),	/* width */
			READ_DATA( sizeof( int ) + 16, GLint ),	/* border */
			READ_DATA( sizeof( int ) + 20, GLenum ),	/* format */
			READ_DATA( sizeof( int ) + 24, GLenum ),	/* type */
			NULL
			 );
	}
	else
	{
		glTexImage1D(
			READ_DATA( sizeof( int ) + 0, GLenum ),	/* target */
			READ_DATA( sizeof( int ) + 4, GLint ),	/* level */
			READ_DATA( sizeof( int ) + 8, GLint ),	/* internalformat */
			READ_DATA( sizeof( int ) + 12, GLsizei ),	/* width */
			READ_DATA( sizeof( int ) + 16, GLint ),	/* border */
			READ_DATA( sizeof( int ) + 20, GLenum ),	/* format */
			READ_DATA( sizeof( int ) + 24, GLenum ),	/* type */
			DATA_POINTER( sizeof( int ) + 32, GLvoid )
			 );
	}
	INCR_VAR_PTR();
}

void __decodeDeleteTextures( void )
{
	GLsizei n = READ_DATA( sizeof( int ) + 0, GLsizei );
	GLuint *clientnames = DATA_POINTER(sizeof( int ) + 4, GLuint );
	int i;

	glDeleteTextures( 
			READ_DATA( sizeof( int ) + 0, GLsizei ), /* n */
			DATA_POINTER( sizeof( int ) + 4, GLuint ) /* textures */
			);

	for (i=0; i<n; i++)
		__gltexture_deletetexture (clientnames[i]);


	INCR_VAR_PTR();
}

void __decodeTexEnvfv( void )
{
	__gltrack_TexEnvfv( 
			READ_DATA( sizeof( int ) + 0, GLenum ), /* target */
			READ_DATA( sizeof( int ) + 4, GLenum ), /* pname */
			DATA_POINTER( sizeof( int ) + 8, GLfloat ) /* params */
			);
	INCR_VAR_PTR();
}

void __decodeTexEnviv( void )
{
	__gltrack_TexEnviv( 
			READ_DATA( sizeof( int ) + 0, GLenum ), /* target */
			READ_DATA( sizeof( int ) + 4, GLenum ), /* pname */
			DATA_POINTER( sizeof( int ) + 8, GLint ) /* params */
			);
	INCR_VAR_PTR();
}

void __decodeTexEnvf( void )
{
	GLenum target = READ_DATA( 0, GLenum );
	GLenum pname = READ_DATA( 4, GLenum );
	GLfloat param = READ_DATA( 8, GLfloat );
	__gltrack_TexEnvf( target, pname, param  );
	INCR_DATA_PTR( 12 );
}

void __decodeTexEnvi( void )
{
	GLenum target = READ_DATA( 0, GLenum );
	GLenum pname = READ_DATA( 4, GLenum );
	GLint param = READ_DATA( 8, GLint );
	__gltrack_TexEnvi( target, pname, param  );
	INCR_DATA_PTR( 12 );
}

void __decodeTexGendv( void )
{
	glTexGendv( 
			READ_DATA( sizeof( int ) + 0, GLenum ), /* coord */
			READ_DATA( sizeof( int ) + 4, GLenum ), /* pname */
			DATA_POINTER( sizeof( int ) + 8, GLdouble ) /* params */
			);
	INCR_VAR_PTR();
}

void __decodeTexGenfv( void )
{
	glTexGenfv( 
			READ_DATA( sizeof( int ) + 0, GLenum ), /* coord */
			READ_DATA( sizeof( int ) + 4, GLenum ), /* pname */
			DATA_POINTER( sizeof( int ) + 8, GLfloat ) /* params */
			);
	INCR_VAR_PTR();
}

void __decodeTexGeniv( void )
{
	glTexGeniv( 
			READ_DATA( sizeof( int ) + 0, GLenum ), /* coord */
			READ_DATA( sizeof( int ) + 4, GLenum ), /* pname */
			DATA_POINTER( sizeof( int ) + 8, GLint ) /* params */
			);
	INCR_VAR_PTR();
}

void __decodePrioritizeTextures( void )
{
	GLsizei n = READ_DATA( sizeof( int ) + 0, GLsizei );
	glPrioritizeTextures( n,
			DATA_POINTER( sizeof( int ) + 4, GLuint ), /* textures */
			DATA_POINTER( sizeof( int ) + 4 + n*sizeof( GLuint ), GLclampf ) /* priorities */
			);
			
	INCR_VAR_PTR();
}

void __decodeBindTexture( void )
{
	GLenum target = READ_DATA(0, GLenum);
	GLuint textureid = READ_DATA(4, GLuint);
	
	__gltrack_BindTexture (target, textureid);

	INCR_DATA_PTR( 8 );
}

void __decodeTexParameterfv( void )
{
	glTexParameterfv(
			READ_DATA( sizeof( int ) + 0, GLenum ), /* target */
			READ_DATA( sizeof( int ) + 4, GLenum ), /* pname */
			DATA_POINTER( sizeof( int ) + 8, GLfloat ) /* params */
			);
	INCR_VAR_PTR();
}

void __decodeTexParameteriv( void )
{
	glTexParameteriv(
			READ_DATA( sizeof( int ) + 0, GLenum ), /* target */
			READ_DATA( sizeof( int ) + 4, GLenum ), /* pname */
			DATA_POINTER( sizeof( int ) + 8, GLint ) /* params */
			); 
	INCR_VAR_PTR();
}

void __decodeTexSubImage2D( void )
{
	glPixelStorei( GL_UNPACK_ROW_LENGTH, 0 );
	glPixelStorei( GL_UNPACK_SKIP_PIXELS, 0 );
	glPixelStorei( GL_UNPACK_SKIP_ROWS, 0 );
	glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
	
	glTexSubImage2D(
		READ_DATA( sizeof( int ) + 0, GLenum ),	/* target */
		READ_DATA( sizeof( int ) + 4, GLint ),	/* level */
		READ_DATA( sizeof( int ) + 8, GLint ),	/* xoffset */
		READ_DATA( sizeof( int ) + 12, GLint ),	/* yoffset */
		READ_DATA( sizeof( int ) + 16, GLsizei ),	/* width */
		READ_DATA( sizeof( int ) + 20, GLsizei ),	/* height */
		READ_DATA( sizeof( int ) + 24, GLenum ),	/* format */
		READ_DATA( sizeof( int ) + 28, GLenum ),	/* type */
		DATA_POINTER( sizeof( int ) + 32, GLvoid ) /* pixels */
	 	 );
	INCR_VAR_PTR();
}

void __decodeTexSubImage1D( void )
{
	glPixelStorei( GL_UNPACK_ROW_LENGTH, 0 );
	glPixelStorei( GL_UNPACK_SKIP_PIXELS, 0 );
	glPixelStorei( GL_UNPACK_SKIP_ROWS, 0 );
	glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
	
	glTexSubImage1D(
		READ_DATA( sizeof( int ) + 0, GLenum ),	/* target */
		READ_DATA( sizeof( int ) + 4, GLint ),	/* level */
		READ_DATA( sizeof( int ) + 8, GLint ),	/* xoffset */
		READ_DATA( sizeof( int ) + 12, GLsizei ),	/* width */
		READ_DATA( sizeof( int ) + 16, GLenum ),	/* format */
		READ_DATA( sizeof( int ) + 20, GLenum ),	/* type */
		DATA_POINTER( sizeof( int ) + 24, GLvoid ) /* pixels */
		 );
	INCR_VAR_PTR();
}
